unit Flic;
interface
uses Objects;
const
  sfeOk      = 0;
  sfeBadMode = -1;
  sfeStream  = -2;
  sfeBadFlic = -3;
  sfeTooBig  = -4;


function ShowFlic ( var Flic : TStream; var Control : boolean ) : integer;
{
  Flic    : stream with flic
  Control : initially FALSE; set to TRUE when ShowFlic has to stop
}

implementation
uses Dos;

const
  sgOldFlic = $AF11;
  sgNewFlic = $AF12;
  sgFrame   = $F1FA;

  btCompHiPal   = $0004;
  btWordDelta   = $0007;
  btCompPalette = $000B;
  btCompDelta   = $000C;
  btEraseScreen = $000D;
  btCompScreen  = $000F;
  btScreen64k   = $0010;

  MaxSize = 65528;

type

  PHeader     = ^THeader;
  THeader     = record
    Size      : longint;
    Signature : word;
  end;

  TFlicHeader = record
    Header    : THeader;
    FrameNo   : word;

    Width     : word;
    Height    : word;
    Some1     : word;
    Some2     : word;

    Speed     : word;
    Some3     : word;
    Some4     : word;
    Some5     : word;
    Pad : array [0..$67] of byte;
  end;

  PFrameHeader = ^TFrameHeader;
  TFrameHeader = record
    Header    : THeader;
    ChunkNo   : word;
    Pad       : array [0..7] of byte;
  end;

  TFrameRec  = record
    Start : longint;
    Size  : longint;
  end;

  PFrameList = ^TFrameList;
  TFrameList = array [0..0] of TFrameRec;

procedure RLE1 (RLE1Src,RLE1Dst : pointer; RLE1LineLen : word); far;
  external;
procedure RLE2 (RLE2Src,RLE2Dst : pointer; RLE2LineLen,RLE2LineCount : word);
  far; external;
procedure RLE3 (RLE3Src, RLE3Dst : pointer; RLE3LineLen : word); far;
  external;
procedure PalBuffered ( PalSrc, PalDst : pointer); far; external;
procedure PalPort (PalSrc : pointer); far; external;
procedure HiPalPort ( HiPalSrc : pointer ); far; external;
procedure SetPal (PalBuffer : pointer; StartColor, CountColor : word); far;
  external;
procedure SetHiPal (HiPalBuffer : pointer; StartColor, CountColor : word); far;
  external;
procedure SetColor0 (Color0 : word); external;
{$L FlicUtil}

var
   OldInt8      : pointer;
   ClockCounter : integer;
   ClockBuf     : word;
   ClockDiv     : word;

procedure Int8; interrupt;
begin
  asm
    inc  ClockCounter
    mov  ax, ClockDiv
    add  ClockBuf, ax
    jnc  @@1
    { call old handler }
    pushf
    call OldInt8
    jmp  @@2
@@1:
    mov  al,20h
    out  20h,al
@@2:
  end;
end;

const
  TimerFreq = 1193180;

procedure SetIntHandler ( Period : word );
var
  Divider : longint;
begin
  Divider := TimerFreq*longint(Period) div 1000;
  if Divider>65535 then
    Divider:=65535;
  ClockDiv:=Divider;
  GetIntVec(8,OldInt8);
  SetIntVec(8,@Int8);
  asm
    cli
    mov  al, 36h
    out  43h, al
    mov  ax, word ptr Divider
    out  40h, al
    mov  al, ah
    out  40h, al
    sti
  end;
end;

procedure ResetIntHandler;
begin
  asm
    cli
    mov  al, 36h
    out  43h, al
    mov  al, 0FFh
    out  40h, al
    nop
    out  40h, al
    sti
  end;
  SetIntVec(8,OldInt8);
end;

function ShowFlic ( var Flic : TStream; var Control : boolean ) : integer;

var
  RetValue : integer;

procedure StreamTest;
begin
  if Flic.Status<>stOk then
    RetValue:=sfeStream;
end;

var
  FlicStart  : longint;
  FlicHeader : TFlicHeader;
  FrameList  : PFrameList;
  FramePos   : longint;
  FrameCnt   : word;
  FrameHeader : TFrameHeader;
  FrameBuf     : PFrameHeader;
  MaxFrameSize : longint;
  ChunkCnt   : word;
  ChunkPtr   : PChar;
  CHPtr      : PHeader;
  ScreenPtr  : pointer;
  Count      : integer;

begin
  RetValue := sfeOk;
  repeat { this is dummy loop -- not to use labels }
    { Test video mode }
    asm
      mov  ah, 0Fh
      int  10h
      cmp  al, 13h
      je   @@1
      mov  RetValue, sfeBadMode
@@1:
    end;
    if RetValue<>sfeOk then
      break;
    ScreenPtr := Ptr ($A000,0);
    { scan flic }
    StreamTest;
    if RetValue<>sfeOk then
      break;
    FlicStart  := Flic.GetPos;
    Flic.Read(FlicHeader,SizeOf(FlicHeader));
    StreamTest;
    if RetValue<>sfeOk then
      break;
    with FlicHeader do
    begin
      if (not ((Header.Signature=sgOldFlic) or
               (Header.Signature=sgNewFlic))) or
         (Width>320) or (Height>200) then
      begin
        RetValue := sfeBadFlic;
        break;
      end;
      GetMem(FrameList,(FrameNo+1)*SizeOf(TFrameRec));
      FrameCnt  := 0;
      MaxFrameSize := 0;
      repeat
        FramePos := Flic.GetPos;
        Flic.Read(FrameHeader, SizeOf(FrameHeader));
        if FrameHeader.Header.Signature=sgFrame then
        begin
          FrameList^[FrameCnt].Start:=FramePos;
          FrameList^[FrameCnt].Size :=FrameHeader.Header.Size;
          if MaxFrameSize<FrameHeader.Header.Size then
            MaxFrameSize:=FrameHeader.Header.Size;
          Inc(FrameCnt);
        end;
        Flic.Seek(FramePos+FrameHeader.Header.Size);
        StreamTest;
      until (FrameCnt=FrameNo+1) or (RetValue<>sfeOk);
      if MaxFrameSize>MaxSize then
        RetValue:=sfeBadFlic;
      if RetValue<>sfeOk then
        break;
      { do the show - everything Ok }
      if Header.Signature = sgOldFlic then
        SetIntHandler ( Speed*1000 div 70 )
      else
        SetIntHandler ( Speed );
      GetMem(FrameBuf,MaxFrameSize);
      FrameCnt := 0;
      Count:=ClockCounter+1;
      repeat
        Flic.Seek(FrameList^[FrameCnt].Start);
        Flic.Read(FrameBuf^,FrameList^[FrameCnt].Size);
        ChunkPtr := PChar(FrameBuf) + SizeOf(TFrameHeader) + SizeOf(THeader);

        repeat
        until (ClockCounter-Count)>=0;

        for ChunkCnt := 1 to FrameBuf^.ChunkNo do
        begin
          CHPtr := PHeader(ChunkPtr - SizeOf(THeader));
          case CHPtr^.Signature of
            btCompHiPal   : HiPalPort(ChunkPtr);
            btWordDelta   : RLE3(ChunkPtr,ScreenPtr,Width);
            btCompPalette : PalPort(ChunkPtr);
            btCompDelta   : RLE1(ChunkPtr,ScreenPtr,Width);
            btEraseScreen : FillChar(ScreenPtr^,64000,0);
            btCompScreen  : RLE2(ChunkPtr,ScreenPtr,Width,Height);
            btScreen64k   : Move(ChunkPtr^,ScreenPtr^,64000);
          end;
          Inc(ChunkPtr,CHPtr^.Size);
        end;

        if Control then
          break;

        FrameCnt:=(FrameCnt +1) mod FrameNo;
        Count:=ClockCounter+1;
      until FALSE;

      ResetIntHandler;
      Flic.Seek(FlicStart);

    end; { with }
    break;
  until FALSE; { this is dummy loop end }
  ShowFlic:=RetValue;
end;

end.
